﻿#region 命名空间
using System;
using System.IO;
using System.Reflection;
using Microsoft.Xna.Framework.Content.Pipeline;
using Microsoft.Xna.Framework.Content.Pipeline.Serialization.Compiler;
using Microsoft.Xna.Framework.Graphics;
using MonoGameContentProcessors;
#endregion

namespace MonoGameContentBuild
{
    /// <summary>
    /// 提供创建素材的方法。
    /// </summary>
    public static class ContentBuildSystem
    {
        #region 字段

        static readonly string tempFolder;

        static readonly CustomContentImporterContext importerContext;// 提供导入程序的上下文对象。
        static readonly CustomContentProcessorContext processorContext;// 提供处理程序的上下文对象。
        static readonly ConstructorInfo compilerConstructor;// 素材编译程序的构造函数。
        static readonly MethodInfo compileMethod;// 提供编译素材的方法。

        #endregion

        #region 初始化

        /// <summary>
        /// 初始化。
        /// </summary>
        static ContentBuildSystem()
        {
            tempFolder = Path.GetTempPath();
            // 初始化导入/处理程序。
            CustomContentBuildLogger loggor = new CustomContentBuildLogger();
            importerContext = new CustomContentImporterContext(loggor);
            processorContext = new CustomContentProcessorContext(loggor);
            // 初始化编译程序。
            Type type = typeof(ContentCompiler);
            ConstructorInfo[] constructorInfoArray = type.GetConstructors(BindingFlags.Instance | BindingFlags.NonPublic | BindingFlags.Public);
            compilerConstructor = constructorInfoArray[0];
            compilerConstructor.Invoke(null);// 第一次实例化耗时长，在这里先执行一次。
            compileMethod = type.GetMethod("Compile", BindingFlags.Instance | BindingFlags.NonPublic | BindingFlags.Public);
        }

        #endregion

        #region 方法

        // 设置目标游戏平台的环境标识。
        static void SetEnvironment(MonoGamePlatform targetPlatform)
        {
            string value = null;
            switch (targetPlatform)
            {
                case MonoGamePlatform.Windows:
                    value = "WINDOWS";
                    break;
                case MonoGamePlatform.Windows8:
                    value = "WINDOWS8";
                    break;
                case MonoGamePlatform.iOS:
                    value = "IOS";
                    break;
                case MonoGamePlatform.Android:
                    value = "ANDROID";
                    break;
                case MonoGamePlatform.Linux:
                    value = "LINUX";
                    break;
                case MonoGamePlatform.OSX:
                    value = "OSX";
                    break;
                case MonoGamePlatform.PSM:
                    value = "PSM";
                    break;
                default:
                    break;
            }
            if (value != null)
                Environment.SetEnvironmentVariable("MONOGAME_PLATFORM", value, EnvironmentVariableTarget.User);
        }

        /// <summary>
        /// 注册用于导入/处理程序的日志记录器。
        /// </summary>
        public static void RegisterLogger(ContentBuildLogger logger)
        {
            if (logger != null)
            {
                lock (importerContext)
                {
                    (importerContext as CustomContentImporterContext).RegisterLogger(logger);
                }
                lock (processorContext)
                {
                    (processorContext as CustomContentProcessorContext).RegisterLogger(logger);
                }
            }
        }

        /// <summary>
        /// 创建素材。
        /// </summary>
        public static void Build(string input, string output, IContentImporter importer, IContentProcessor processor, MonoGamePlatform targetPlatform, bool compressContent, ContentBuildCallback completed = null)
        {
            object content = Import(input, importer);
            content = Process(output, content, processor, targetPlatform);
            Compile(output, content, compressContent);
            if (completed != null)
                completed(input);
        }

        /// <summary>
        /// 导入指定素材文件。
        /// </summary>
        public static object Import(string fileName, IContentImporter importer)
        {
            if (importer != null)
            {
                lock (importerContext)
                {
                    return importer.Import(fileName, importerContext);
                }
            }
            else
                return null;
        }

        /// <summary>
        /// 处理指定素材对象。
        /// </summary>
        public static object Process(string fileName, object content, IContentProcessor processor, MonoGamePlatform targetPlatform)
        {
            if (processor != null)
            {
                lock (processorContext)
                {
                    processorContext.SetOutput(fileName);
                    SetEnvironment(targetPlatform);
                    return processor.Process(content, processorContext);
                }
            }
            else
                return content;
        }

        /// <summary>
        /// 编译素材到指定路径。
        /// </summary>
        public static void Compile(string fileName, object content, bool compressContent)
        {
            string directory = Path.GetDirectoryName(fileName);
            if (!Directory.Exists(directory))
            {
                Directory.CreateDirectory(directory);
            }
            using (Stream stream = new FileStream(fileName, FileMode.Create))
            {
                object[] parameters = new object[]{
                    stream,
                    content,
                    TargetPlatform.Windows,
                    GraphicsProfile.HiDef,
                    compressContent,
                    tempFolder,
                    tempFolder,
                };
                ContentCompiler contentCompiler = compilerConstructor.Invoke(null) as ContentCompiler;
                compileMethod.Invoke(contentCompiler, parameters);
            }
        }

        #endregion
    }
}
